def product_description(competency, names):
    """return scenario description and example query for a specified product"""
    description = ""
    example_queries = ""
    if competency == "get_transactions":
        description = f"User inquires about {names[-1]} account transactions."
        example_queries = f"What are the transactions on my {names[-1]} account?"
    elif competency == "faq_open_account":
        description = f"User requests to open about a {names[-1]} account."
        example_queries = f"Can I open a {names[-1]} account?"
    else:
        account_type = names[0].split("_")
        if account_type[-1] in ["rates", "fees"]:
            description = f"User inquires about {names[-1]} account {account_type[-1]}."
            example_queries = f"What are the {names[-1]} account {account_type[-1]}?"
        else:
            description = f"User inquires about {names[-1]} accounts."
            example_queries = f"Tell me about {names[-1]} accounts."
    return (description, example_queries)


def build_describe_accounts_cases(names):
    return [
        {"name": "subtype", "values": [f"{names[0]}"]},
        {"name": "type_of_account", "values": [f"{names[1]}"]},
    ]


def build_get_transaction_cases(names):
    return [
        {"name": "trxn_spnd_hist_filter", "values": [f"{names[0]}"]},
    ]


def build_faq_open_account_cases(names):
    return [{"name": "type_of_account", "values": [f"{names[0]}"]}]


def get_special_products_competencies(product):
    return [
        {
            "competency": "faq_describe_accounts",
            "cases": {
                "subtype": "faq_describe_accounts",
                "type_of_account": product,
            },
        },
        {
            "competency": "faq_describe_accounts",
            "cases": {
                "subtype": "faq_product_rates",
                "type_of_account": product,
            },
        },
        {
            "competency": "faq_describe_accounts",
            "cases": {
                "subtype": "faq_product_fees",
                "type_of_account": product,
            },
        },
        {
            "competency": "faq_open_account",
            "cases": {"type_of_account": product},
        },
        {
            "competency": "get_transactions",
            "cases": {"trxn_spnd_hist_filter": product},
        },
    ]


def build_cases(competency, names: list):
    if competency == "faq_describe_accounts":
        return build_describe_accounts_cases(names)
    if competency == "get_transactions":
        return build_get_transaction_cases(names)
    if competency == "faq_open_account":
        return build_faq_open_account_cases(names)


def get_product_scenario(competency, description, example_queries, names):
    return {
        "name": ": ".join(names),
        "description": description,
        "example_queries": [example_queries],
        "disabled": True,
        "tags": ["new_scenario"],
        "cases": build_cases(competency, names),
        "response_elements": [
            {
                "type": "paragraph",
                "speakable_fallback_behavior": "default",
                "values": [
                    {
                        "value": f"Thanks for asking! We offer a variety of {names[-1]} accounts to fit your needs. Click below to learn more.",
                        "display": "",
                        "speakable": "",
                    }
                ],
            },
            {
                "type": "url",
                "speakable_fallback_behavior": "default",
                "values": [
                    {
                        "value": f'https://example.com/{names[-1].lower().replace(" ", "-")}',
                        "display": f"{names[-1]}",
                        "speakable": "",
                    }
                ],
            },
        ],
    }



import csv
import json

from scribe_updater.product import (
    get_product_scenario,
    get_special_products_competencies,
    product_description,
)


def read_csv(filename: str):
    """
    Reads csv specified at filename
    """
    with open(filename, "r") as csvreader:
        return list(csv.DictReader(csvreader))


def load_json(file):
    """
    Reads json file and stores into a dict
    """
    if not file:
        return {}

    with open(file, "r") as f:
        f = json.loads(f.read())

    return f


def get_scenarios(info):
    """
    creates a dictionary of scenarios
    """
    ids = {}

    if "competencies" not in info:
        return ids

    for i in info["competencies"]:
        s = {}
        if not i["scenarios"]:
            ids[i["name"]] = []
        else:
            for scenario in i["scenarios"]:
                s["{}".format(scenario["name"])] = scenario
            ids[i["name"]] = s
    return ids


def get_competencies(info):
    """
    Creates a dictionary of competencies
    """
    competencies = {}

    if "competencies" not in info:
        return competencies

    for i in info["competencies"]:
        comp = {
            "name": i["name"],
            "type": i["type"],
            "disabled": i["disabled"],
            "templates": i["templates"],
        }
        competencies[i["name"]] = comp
    return competencies


def merge_scenarios(
    scenario_name, ground_scenarios, target_scenarios, target_scenario_name
):
    """Used to merge two scenarios"""
    scenario = {}
    fields = [
        "name",
        "description",
        "example_queries",
        "disabled",
        "tags",
        "cases",
        "response_elements",
    ]
    for field in fields:
        scenario[field] = (
            target_scenarios[target_scenario_name][field]
            if (field == "disabled" or field == "response_elements")
            and (
                target_scenarios.get(target_scenario_name)
                and target_scenarios[target_scenario_name].get(field)
            )
            else ground_scenarios[scenario_name][field]
        )
    return scenario


def create_scenario(
    c, scenario_name, ground_scenarios, all_target_scenarios, finie_to_fi_map
):
    """
    Creates a new scenario
    """
    scenario = {}

    fields = [
        "name",
        "description",
        "example_queries",
        "disabled",
        "tags",
        "cases",
        "response_elements",
    ]
    for field in fields:
        if f"{c}-*-{scenario_name}" in finie_to_fi_map.keys():
            fi_competency_name = finie_to_fi_map[f"{c}-*-{scenario_name}"].split("-*-")[
                0
            ]
            fi_scenario_name = finie_to_fi_map[f"{c}-*-{scenario_name}"].split("-*-")[1]
            return merge_scenarios(
                scenario_name,
                ground_scenarios,
                all_target_scenarios[fi_competency_name],
                fi_scenario_name,
            )
        else:
            if field == "tags":
                scenario[field] = ground_scenarios[scenario_name][field] + [
                    "new_scenario"
                ]
            elif field == "disabled":
                scenario[field] = True
            else:
                scenario[field] = ground_scenarios[scenario_name][field]
    return scenario


def parse_product_name(name: str):
    names = name.split(": ")
    return [name.strip() for name in names]


def build_product_scenario(competency, names: list):
    (description, example_queries) = product_description(competency, names)
    return get_product_scenario(competency, description, example_queries, names)


def create_product_scenario(c, scenario_name, target_scenarios, is_new):
    """
    Creates a new product scenario
    """
    if is_new:
        names = parse_product_name(scenario_name)
        scenario = build_product_scenario(c, names)
        return scenario

    scenario = {}

    fields = [
        "name",
        "description",
        "example_queries",
        "disabled",
        "tags",
        "cases",
        "response_elements",
    ]
    for field in fields:
        scenario[field] = target_scenarios[scenario_name][field]
    return scenario


def process_scenario_list(
    competency,
    ground_scenarios,
    target_scenarios,
    products,
    finie_to_fi_map,
    is_glia_vba=False,
):
    """
    Wrapper for handling the scenarios
    """
    product_list = [
        "faq_describe_accounts",
        "faq_open_account",
        "get_transactions",
    ]
    scenarios = []

    if not ground_scenarios[competency]:
        return scenarios

    for scenario in ground_scenarios[competency].keys():

        if "straight bussin" in scenario:
            continue

        if (
            not target_scenarios.get(competency)
            or scenario not in target_scenarios[competency].keys()
        ):
            updated_scenario = create_scenario(
                competency,
                scenario,
                ground_scenarios[competency],
                target_scenarios,
                finie_to_fi_map,
            )
        else:
            updated_scenario = merge_scenarios(
                scenario,
                ground_scenarios[competency],
                target_scenarios[competency],
                scenario,
            )
        if is_glia_vba:
            for response_element in updated_scenario["response_elements"]:
                if (
                    response_element["type"] == "url"
                    and response_element["values"][0]["display"] == "Login"
                ):
                    response_element["values"][0]["value"] = "~glia#login"
        scenarios.append(updated_scenario)
    if not products:
        return scenarios

    if competency in product_list:
        products_lower = [p.lower() for p in products[competency]]
        for scenario in target_scenarios[competency].keys():
            if scenario.lower().strip() in products_lower:
                scenarios.append(
                    create_product_scenario(
                        competency,
                        scenario,
                        target_scenarios[competency],
                        False,
                    )
                )
                # removes scenario, anything that remains should
                # have a new special product created
                products[competency].remove(scenario)

        # loop through and add remaining products as new scenarios
        for remaining_scenario in products[competency]:
            scenarios.append(
                create_product_scenario(competency, remaining_scenario, {}, True)
            )

    scenarios[1:] = sorted(scenarios[1:], key=lambda s: s["name"])
    return scenarios


def process_competency(
    competency_name,
    target_competencies,
    ground_competencies,
    products,
    ground_scenarios,
    target_scenarios,
    finie_to_fi_map,
    is_glia_vba=False,
):
    """
    Process a competency
    """
    competency = {}
    competency["name"] = competency_name
    competency["type"] = ground_competencies[competency_name]["type"]
    competency["disabled"] = (
        target_competencies[competency_name]["disabled"]
        if competency_name in target_competencies
        else True
    )
    competency["scenarios"] = process_scenario_list(
        competency_name,
        ground_scenarios,
        target_scenarios,
        products,
        finie_to_fi_map,
        is_glia_vba=is_glia_vba,
    )
    competency["templates"] = ground_competencies[competency_name]["templates"]
    return competency


def make_lower(data: dict):
    if not data:
        return {}

    for competency in data["competencies"]:
        for scenario in competency["scenarios"]:
            scenario["name"] = scenario["name"].lower()

    return data


def process_variable_rows_csv(variables):
    """
    Processes the rows of the CSV for the Variables sheet into a list of variables
    """
    v = []
    for row in variables:

        value: str = row["Value"]

        if value in ["TRUE", "FALSE"]:
            value = value.lower()

        v.append(
            {
                "name": row["Name"],
                "value": value,
            }
        )

    return {"variables": v}


def process_variable_rows_json(variables):
    """
    Processes the variables.json file, it is similar the the process_variable_rows_csv function
    """
    v = []

    for row in variables.get("variables"):
        value: str = row["value"]
        if value.lower() in ["true", "false"]:
            value = value.lower()

        v.append(
            {
                "name": row["name"],
                "value": value,
            }
        )
    return {"variables": v}


def get_variable(variables: dict, variable_name):
    v = variables.get("variables", [])
    for variable in v:
        if variable.get("name", "") == variable_name:
            return variable.get("value", "")

    return ""


def create_product_names(products: list):
    product_names = {
        "faq_describe_accounts": [],
        "faq_open_account": [],
        "get_transactions": [],
    }
    for product in products:
        target = get_special_products_competencies(product)

        for i in target:
            case_names = []
            cases: dict = i["cases"]
            for _, v in cases.items():
                case_names.append(v.strip().lower())

            product_names[i["competency"]].append(": ".join(case_names))

    return product_names


def get_product_names(variables):
    institution_products = get_variable(variables, "institution_products")
    institution_products = institution_products.split(",")
    product_names = create_product_names(institution_products)
    return product_names


def get_glia_vba(variables):
    is_glia_vba = get_variable(variables, "is_glia_vba")
    return True if is_glia_vba and is_glia_vba != "false" else False



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['scribe_updater']

package_data = \
{'': ['*'], 'scribe_updater': ['version_mapping/*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'nox>=2022.11.21,<2023.0.0']

entry_points = \
{'console_scripts': ['updater = scribe_updater.console:main']}

setup_kwargs = {
    'name': 'scribe-updater',
    'version': '0.17.0',
    'description': 'A tool to upgrade scribe configuration files',
    'long_description': "### **Setup**\npoetry runs a venv, so you need to run `poetry install` from the root of the repo first\n\n#### **Run**\n`poetry run updater`\n\n#### **Test**\n`poetry run pytest` \n\n#### **Coverage**\n`poetry run pytest --cov` or `poetry run pytest --cov --cov-report=term-missing`\n\n### **Publish to PyPi**\nAfter adding credentials to be able to push to the python package index run the following cmd:\n`poetry publish --build`\n\n##### **Linting**\nInstall nox with:\n`pip3 install nox`\nadd the path to your .bashrc and source it\n\nrun `nox -rs black`\n##### **Testing with console.py**\n\nfrom within the src/scribe_updater/ directory run the following:\n`poetry run updater --target ../tests/test_input_1.json --ground ../tests/test_ground_1.json --output ./test_output`\n'\n##### **PBA Customer Spreadsheet Updater (`update.py`)**\nInjects new scenarios from the ground truth into a customer spreadsheet.\n##### **Caveats**\nif you are getting an error that looks like this :<br> `Failed to create the collection: Prompt dismissed..`<br>\nthen export the following environment variable: <br>\n`export PYTHON_KEYRING_BACKEND=keyring.backends.null.Keyring`\n\n### **Usage**\n\nbasic use with no variables or mapping<br>\n`updater -t target.json -g ground.json -o output.json`\n\nuse with variables (variables are only required when adding special products)<br>\n`updater -t target.json -g ground.json -v variables.json -o output.json`\n\nuse with version map<br>\n`updater -t target.json -g ground.json -m 3.3_to_3.7.json -o output.json`\n\nuse with variables & version maps<br>\n`updater -t target.json -g ground.json -v variables -m 3.3_to_3.7.json -o output.json`\n\n\n### **Version Mappings**\n\nVersion mappings are necessary when upgrading to a version that has scenarios that still exist but have been moved or their names have been changed. If you omit the version map the updater will still work but may delete or add scenarios that should not be in the configuration.\n\n#### Available Mappings\n<ol>\n    <li>3.3_to_3.7.json (use this to upgrade to finie 3.8)</li>\n    <li>3.3_to_3.6.json</li>\n    <li>3.3_to_3.5.json (use this to upgrade to finie 3.8)</li>\n</ol>",
    'author': 'Paul Cardoos',
    'author_email': 'paul.cardoos@clinc.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import json
import os

from scribe_updater.changelog import ChangeLog
from scribe_updater.utils import (
    get_competencies,
    get_glia_vba,
    get_product_names,
    get_scenarios,
    process_competency,
    process_variable_rows_csv,
    process_variable_rows_json,
)


class Updater:
    def __init__(
        self,
        ground,
        target,
        variables,
        output_file_path,
        ground_to_target_map,
        is_csv=False,
    ):
        self.ground = ground
        self.target = target
        self.variables = variables
        self.target_scenarios = {}
        self.ground_scenarios = {}
        self.target_competencies = {}
        self.ground_competencies = {}
        self.products = {}
        self.output = {}
        self.output_file_path = output_file_path
        self.ground_to_target_map = ground_to_target_map
        self.is_csv = is_csv

    def set_ground_scenarios(self):
        """
        Set scenarios from the ground truth
        """
        self.ground_scenarios = get_scenarios(self.ground)

    def set_target_scenarios(self):
        """
        Set scenarios from the input ground truth (ex. Landmark Ground Truth)
        """
        self.target_scenarios = get_scenarios(self.target)

    def set_ground_competencies(self):
        """
        Set competencies from the ground truth
        """
        self.ground_competencies = get_competencies(self.ground)

    def set_target_competencies(self):
        """
        Set competencies from the input ground truth (ex. Landmark Ground Truth)
        """
        self.target_competencies = get_competencies(self.target)

    def set_variables(self):
        """
        Set the relevant variables for the individual fi
        """
        if not self.variables:
            self.is_glia_vba = False
            return

        v = (
            process_variable_rows_json(self.variables)
            if not self.is_csv
            else process_variable_rows_csv(self.variables)
        )
        products = get_product_names(v)
        self.products = products
        self.is_glia_vba = get_glia_vba(v)

    def init(self):
        """
        Initialize all the data from the ground truths both input and master
        """
        self.set_ground_competencies()
        self.set_ground_scenarios()
        self.set_target_competencies()
        self.set_target_scenarios()
        self.set_variables()

    def prep_output_pba(self):
        """
        preparation function for pba output
        """
        self.output = {
            "auth": "pre",
            "competencies": [],
            "disabled_response": self.target["disabled_response"],
            "outofscope_response": self.target["outofscope_response"],
        }

    def prep_output_vba(self):
        """
        preparation function for vba output
        """
        self.output = {"auth": "post", "competencies": []}

    def update(self):
        """
        Main logic is performed here to create an output file.
        The newly generated output file can be used as the new
        master ground truth
        """
        self.init()
        self.output["competencies"] = [
            process_competency(
                competency,
                self.target_competencies,
                self.ground_competencies,
                self.products,
                self.ground_scenarios,
                self.target_scenarios,
                self.ground_to_target_map,
                is_glia_vba=self.is_glia_vba,
            )
            for competency in self.ground_competencies.keys()
        ]

    def output_to_json(self):
        """
        Takes the output formed from the update functions and prints the results as json
        """
        if not os.path.isdir("results"):
            os.mkdir("results")

        with open("results/" + self.output_file_path, "w") as f:
            f.write(json.dumps(self.output, indent=4))

    def get_change_log(self):
        """
        Creates a change log that can be views in the terminal
        """
        change_log = ChangeLog(
            self.ground_scenarios,
            self.target_scenarios,
            self.ground_competencies,
            self.target_competencies,
            self.output,
            self.ground_to_target_map,
        )
        log = change_log.create_change_log()
        change_log.write_change_log_to_file()
        change_log.display_change_log()
        return log
